#! /usr/bin/env ruby

require 'optionparser'
require 'shellwords'
require 'pathname'

require_relative 'textpatch'

# Note: in kernel overlayfs ignores bind mounts in source
FUSE = true

MOUNT_BIND = %w(sudo mount -o bind)
MOUNT_OVERLAY = FUSE ? %w(fuse-overlayfs) : %w(sudo mount -t overlay)
UMOUNT = false ? %w(fusermount -u) : %w(sudo umount)

def cmd *args
  $stderr.puts "+ #{args.shelljoin}"
  system *args
  $?.success?
end

def umount dir
  cmd('mountpoint', '-q', dir) && (
    cmd(*UMOUNT, dir) || fail('Umount failed'))
end

Dir.chdir __dir__
base = 'ovrlay'

dirs = [File.join(base, 'base')]
cars = Dir.glob('*', base: File.join(base, 'mod/car')).to_set
misc = Dir.glob('*', base: File.join(base, 'mod/misc')).to_set

tr_file = 'text/unicodetext.eng'
translate = NFS6Text.read File.read File.join base, 'base', tr_file

save = true
OptionParser.new do |opt|
  opt.on('--no-save') { save = false }
end.parse!

car_dirs = {}
ARGV.each do |a|
  if a =~ /\A([^=]+)=(.+)\z/ && NFS6Text::CAR_MAP[$1] && cars.member?($2)
    id, name = $1, $2
    fail "Duplicate car #{id}" if car_dirs[id]
    info = NFS6Text::CAR_MAP[id]
    dir = File.join base, 'mod/car', name

    cap = begin
            File.read(File.join dir, 'name').chomp
          rescue Errno::ENOENT
            name.split(/[ _]/).map(&:capitalize).join ' '
          end
    info[0].each {|i| translate[i] = cap }
    info[1].each {|i| translate[i] = "NFS #{cap}" }
    car_dirs[id] = dir
  elsif misc.member? a
    dirs.unshift File.join(base, 'mod/misc', a)
  else
    fail "Unknown mod #{a}"
  end
end

dirs.unshift File.join(base, 'save') if save

File.open File.join(base, 'save', tr_file), 'w' do |f|
  NFS6Text.write translate, f
end

if dirs.size >= 2
  dirs.map! {|d| d.gsub %r{([\\,:])}, '\\\1' }
  work = File.join base, 'idiotism'
  # note: a single mount option has an undocumented length limit, enough to reach
  # by 3-4 mods... lowerdir+ works, but has weird syntax
  ld = FUSE ? "lowerdir=#{dirs[1...].join ':'}," :
         dirs[1...].map {|x| "lowerdir+=#{x}," }.join
  opts = "#{ld}upperdir=#{dirs[0]},workdir=#{work}"
end
tmp_mnt = File.join(base, 'mount')
real_mnt = 'mount'

NFS6Text::CAR_MAP.each {|k,_| umount File.join base, 'base/cars', k }
umount real_mnt
umount tmp_mnt

car_dirs.each do |id,tgt|
  cmd(*MOUNT_BIND, tgt, File.join(base, 'base/cars', id)) ||
    fail('Bind mount failed')
end

cmd(*MOUNT_OVERLAY, '-o', opts, 'nothing', tmp_mnt) ||
  fail('Tmp mount failed') if opts

cmd('mountpoint', '-q', real_mnt) ||
  cmd('ciopfs', opts ? tmp_mnt : dirs[0], real_mnt) ||
  fail('Real mout failed')
